@@ -158,38 +158,35 @@ Hint: A good and simple activity to learn from is the Read activity.
 You may copy it and use it as a template.
 '''
 
+import six
 import gettext
 import logging
 import os
+import signal
 import time
 from hashlib import sha1
 from functools import partial
-import StringIO
 import cairo
 import json
 
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('Gdk', '3.0')
+gi.require_version('TelepathyGLib', '0.12')
 gi.require_version('SugarExt', '1.0')
 
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gdk
 from gi.repository import Gtk
+from gi.repository import TelepathyGLib
 import dbus
 import dbus.service
 from dbus import PROPERTIES_IFACE
-from telepathy.server import DBusProperties
-from telepathy.interfaces import CHANNEL, \
-    CHANNEL_TYPE_TEXT, \
-    CLIENT, \
-    CLIENT_HANDLER
-from telepathy.constants import CONNECTION_HANDLE_TYPE_CONTACT
-from telepathy.constants import CONNECTION_HANDLE_TYPE_ROOM
 
 from sugar3 import util
 from sugar3 import power
-from sugar3.profile import get_nick_name, get_color, get_save_as
+from sugar3.profile import get_color, get_save_as
 from sugar3.presence import presenceservice
 from sugar3.activity.activityservice import ActivityService
 from sugar3.graphics import style
@@ -204,7 +201,10 @@ from errno import EEXIST
 
 from gi.repository import SugarExt
 
-_ = lambda msg: gettext.dgettext('sugar-toolkit-gtk3', msg)
+
+def _(msg):
+    return gettext.dgettext('sugar-toolkit-gtk3', msg)
+
 
 SCOPE_PRIVATE = 'private'
 SCOPE_INVITE_ONLY = 'invite'  # shouldn't be shown in UI, it's implicit
@@ -218,6 +218,14 @@ N_BUS_NAME = 'org.freedesktop.Notifications'
 N_OBJ_PATH = '/org/freedesktop/Notifications'
 N_IFACE_NAME = 'org.freedesktop.Notifications'
 
+CHANNEL = TelepathyGLib.IFACE_CHANNEL
+CHANNEL_TYPE_TEXT = TelepathyGLib.IFACE_CHANNEL_TYPE_TEXT
+CLIENT = TelepathyGLib.IFACE_CLIENT
+CLIENT_HANDLER = TelepathyGLib.IFACE_CLIENT_HANDLER
+
+CONNECTION_HANDLE_TYPE_CONTACT = TelepathyGLib.HandleType.CONTACT
+CONNECTION_HANDLE_TYPE_ROOM = TelepathyGLib.HandleType.ROOM
+
 CONN_INTERFACE_ACTIVITY_PROPERTIES = 'org.laptop.Telepathy.ActivityProperties'
 
 PREVIEW_SIZE = style.zoom(300), style.zoom(225)
@@ -281,16 +289,26 @@ class Activity(Window, Gtk.Container):
     Initialise an Activity.
 
     Args:
-        handle (:class:`~sugar3.activity.activityhandle.ActivityHandle`): instance providing the activity id and access to the presence service which *may* provide sharing for this application
-        create_jobject (boolean): DEPRECATED: define if it should create a journal object if we are not resuming. The parameter is ignored, and always will be created a object in the Journal.
+        handle (:class:`~sugar3.activity.activityhandle.ActivityHandle`):
+            instance providing the activity id and access to the presence
+            service which *may* provide sharing for this application
+
+        create_jobject (boolean):
+            DEPRECATED: define if it should create a journal object if
+            we are not resuming. The parameter is ignored, and always
+            will be created a object in the Journal.
 
     **Signals:**
-        * **shared** - the activity has been shared on a network in order that other users may join,
-        * **joined** - the activity has joined with other instances of the activity to create a shared network activity.
+        * **shared** - the activity has been shared on a network in
+            order that other users may join,
+
+        * **joined** - the activity has joined with other instances of
+            the activity to create a shared network activity.
 
     Side effects:
 
-        * sets the gdk screen DPI setting (resolution) to the Sugar screen resolution.
+        * sets the gdk screen DPI setting (resolution) to the Sugar
+          screen resolution.
 
         * connects our "destroy" message to our _destroy_cb method.
 
@@ -314,6 +332,10 @@ class Activity(Window, Gtk.Container):
     }
 
     def __init__(self, handle, create_jobject=True):
+        if hasattr(GLib, 'unix_signal_add'):
+            GLib.unix_signal_add(
+                GLib.PRIORITY_DEFAULT, signal.SIGINT, self.close)
+
         # Stuff that needs to be done early
         icons_path = os.path.join(get_bundle_path(), 'icons')
         Gtk.IconTheme.get_default().append_search_path(icons_path)
@@ -353,6 +375,8 @@ class Activity(Window, Gtk.Container):
         self.connect('realize', self.__realize_cb)
         self.connect('delete-event', self.__delete_event_cb)
 
+        self._in_main = False
+        self._iconify = False
         self._active = False
         self._active_time = None
         self._spent_time = 0
@@ -460,7 +484,16 @@ class Activity(Window, Gtk.Container):
         """
         self._stop_buttons.append(button)
 
+    def iconify(self):
+        if not self._in_main:
+            self._iconify = True  # i.e. do after Window.show()
+        else:
+            Window.iconify(self)
+
     def run_main_loop(self):
+        if self._iconify:
+            Window.iconify(self)
+        self._in_main = True
         Gtk.main()
 
     def _initialize_journal_object(self):
@@ -578,7 +611,7 @@ class Activity(Window, Gtk.Container):
             if not self._active and self._jobject:
                 self.save()
 
-    active = GObject.property(
+    active = GObject.Property(
         type=bool, default=False, getter=get_active, setter=set_active)
     '''
         Whether an activity is active.
@@ -617,7 +650,7 @@ class Activity(Window, Gtk.Container):
         '''
         self._max_participants = participants
 
-    max_participants = GObject.property(
+    max_participants = GObject.Property(
         type=int, default=0, getter=get_max_participants,
         setter=set_max_participants)
 
@@ -721,25 +754,8 @@ class Activity(Window, Gtk.Container):
         '''
         Deprecated. This part of the API has been moved
         out of this class to the module itself
-
-        Returns:
-            str: a path for saving Activity specific preferences, etc.
-
-        Returns a path to the location in the filesystem where the activity can
-        store activity related data that doesn't pertain to the current
-        execution of the activity and thus cannot go into the DataStore.
-
-        Currently, this will return something like
-        ~/.sugar/default/MyActivityName/
-
-        Activities should ONLY save settings, user preferences and other data
-        which isn't specific to a journal item here. If (meta-)data is in
-        anyway specific to a journal entry, it MUST be stored in the DataStore.
         '''
-        if os.environ.get('SUGAR_ACTIVITY_ROOT'):
-            return os.environ['SUGAR_ACTIVITY_ROOT']
-        else:
-            return get_activity_root()
+        return get_activity_root()
 
     def read_file(self, file_path):
         '''
@@ -852,6 +868,9 @@ class Activity(Window, Gtk.Container):
             return None
 
         window = self.canvas.get_window()
+        if window is None:
+            return None
+
         alloc = self.canvas.get_allocation()
 
         dummy_cr = Gdk.cairo_create(window)
@@ -889,7 +908,7 @@ class Activity(Window, Gtk.Container):
         cr.set_source_surface(screenshot_surface)
         cr.paint()
 
-        preview_str = StringIO.StringIO()
+        preview_str = six.BytesIO()
         preview_surface.write_to_png(preview_str)
         return preview_str.getvalue()
 
@@ -898,7 +917,10 @@ class Activity(Window, Gtk.Container):
             buddies = {}
             for buddy in self.shared_activity.get_joined_buddies():
                 if not buddy.props.owner:
-                    buddy_id = sha1(buddy.props.key).hexdigest()
+                    if six.PY2:
+                        buddy_id = sha1(buddy.props.key).hexdigest()
+                    else:
+                        buddy_id = sha1((buddy.props.key).encode('utf-8')).hexdigest()
                     buddies[buddy_id] = [buddy.props.nick, buddy.props.color]
             return buddies
         else:
@@ -927,7 +949,7 @@ class Activity(Window, Gtk.Container):
 
         buddies_dict = self._get_buddies()
         if buddies_dict:
-            self.metadata['buddies_id'] = json.dumps(buddies_dict.keys())
+            self.metadata['buddies_id'] = json.dumps(list(buddies_dict.keys()))
             self.metadata['buddies'] = json.dumps(self._get_buddies())
 
         # update spent time before saving
@@ -1089,8 +1111,8 @@ class Activity(Window, Gtk.Container):
         '''
         self._invites_queue.append((account_path, contact_id))
 
-        if (self.shared_activity is None
-                or not self.shared_activity.props.joined):
+        if (self.shared_activity is None or
+                not self.shared_activity.props.joined):
             self.share(True)
         else:
             self._send_invites()
@@ -1111,8 +1133,9 @@ class Activity(Window, Gtk.Container):
             raise RuntimeError('Activity %s already shared.' %
                                self._activity_id)
         verb = private and 'private' or 'public'
-        logging.debug('Requesting %s share of activity %s.' % (verb,
-                      self._activity_id))
+        logging.debug(
+            'Requesting %s share of activity %s.' %
+            (verb, self._activity_id))
         pservice = presenceservice.get_instance()
         pservice.connect('activity-shared', self.__share_cb)
         pservice.share_activity(self, private=private)
@@ -1205,8 +1228,8 @@ class Activity(Window, Gtk.Container):
         if response_id == Gtk.ResponseType.OK:
             title = alert.entry.get_text()
             if self._is_resumed and \
-                title == self._original_title:
-                    datastore.delete(self._jobject_old.get_object_id())
+                    title == self._original_title:
+                datastore.delete(self._jobject_old.get_object_id())
             self._jobject.metadata['title'] = title
             self._do_close(False)
 
@@ -1230,7 +1253,7 @@ class Activity(Window, Gtk.Container):
         label = _('Save new')
         tip = _('Save a new journal entry')
         if self._is_resumed and \
-            title == self._original_title:
+                title == self._original_title:
             label = _('Save')
             tip = _('Save into the old journal entry')
 
@@ -1252,7 +1275,7 @@ class Activity(Window, Gtk.Container):
         if not skip_save:
             try:
                 self.save()
-            except:
+            except BaseException:
                 # pylint: disable=W0702
                 logging.exception('Error saving activity object to datastore')
                 self._show_keep_failed_dialog()
@@ -1314,12 +1337,12 @@ class Activity(Window, Gtk.Container):
 
     def __realize_cb(self, window):
         display_name = Gdk.Display.get_default().get_name()
-        if ':' in display_name: 
+        if ':' in display_name:
             # X11 for sure; this only works in X11
             xid = window.get_window().get_xid()
             SugarExt.wm_set_bundle_id(xid, self.get_bundle_id())
             SugarExt.wm_set_activity_id(xid, str(self._activity_id))
-        elif display_name is 'Broadway': 
+        elif display_name is 'Broadway':
             # GTK3's HTML5 backend
             # This is needed so that the window takes the whole browser window
             self.maximize()
@@ -1412,7 +1435,7 @@ class Activity(Window, Gtk.Container):
         Gdk.flush()
 
 
-class _ClientHandler(dbus.service.Object, DBusProperties):
+class _ClientHandler(dbus.service.Object):
     def __init__(self, bundle_id, got_channel_cb):
         self._interfaces = set([CLIENT, CLIENT_HANDLER, PROPERTIES_IFACE])
         self._got_channel_cb = got_channel_cb
@@ -1423,12 +1446,13 @@ class _ClientHandler(dbus.service.Object, DBusProperties):
 
         path = '/' + name.replace('.', '/')
         dbus.service.Object.__init__(self, bus_name, path)
-        DBusProperties.__init__(self)
 
-        self._implement_property_get(CLIENT, {
+        self._prop_getters = {}
+        self._prop_setters = {}
+        self._prop_getters.setdefault(CLIENT, {}).update({
             'Interfaces': lambda: list(self._interfaces),
         })
-        self._implement_property_get(CLIENT_HANDLER, {
+        self._prop_getters.setdefault(CLIENT_HANDLER, {}).update({
             'HandlerChannelFilter': self.__get_filters_cb,
         })
 
@@ -1440,7 +1464,7 @@ class _ClientHandler(dbus.service.Object, DBusProperties):
         }
         filter_dict = dbus.Dictionary(filters, signature='sv')
         logging.debug('__get_filters_cb %r' % dbus.Array([filter_dict],
-                      signature='a{sv}'))
+                                                         signature='a{sv}'))
         return dbus.Array([filter_dict], signature='a{sv}')
 
     @dbus.service.method(dbus_interface=CLIENT_HANDLER,
@@ -1456,9 +1480,39 @@ class _ClientHandler(dbus.service.Object, DBusProperties):
                 handle_type = properties[CHANNEL + '.TargetHandleType']
                 if channel_type == CHANNEL_TYPE_TEXT:
                     self._got_channel_cb(connection, object_path, handle_type)
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
 
+    @dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE,
+                         in_signature='ss', out_signature='v')
+    def Get(self, interface_name, property_name):
+        if interface_name in self._prop_getters \
+           and property_name in self._prop_getters[interface_name]:
+                return self._prop_getters[interface_name][property_name]()
+        else:
+            logging.debug('InvalidArgument')
+
+    @dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE,
+                         in_signature='ssv', out_signature='')
+    def Set(self, interface_name, property_name, value):
+        if interface_name in self._prop_setters \
+           and property_name in self._prop_setters[interface_name]:
+                self._prop_setters[interface_name][property_name](value)
+        else:
+            logging.debug('PermissionDenied')
+
+    @dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE,
+                         in_signature='s', out_signature='a{sv}')
+    def GetAll(self, interface_name):
+        if interface_name in self._prop_getters:
+            r = {}
+            for k, v in list(self._prop_getters[interface_name].items()):
+                r[k] = v()
+            return r
+        else:
+            logging.debug('InvalidArgument')
+
+
 _session = None
 
 
@@ -1491,6 +1545,19 @@ def get_activity_root():
     '''
     Returns:
         str: a path for saving Activity specific preferences, etc.
+
+    Returns a path to the location in the filesystem where the
+    activity can store activity related data that doesn't pertain to
+    the current execution of the activity and thus cannot go into the
+    DataStore.
+
+    Currently, this will return something like
+    ~/.sugar/default/MyActivityName/
+
+    Activities should ONLY save settings, user preferences and other
+    data which isn't specific to a journal item here. If (meta-)data
+    is in anyway specific to a journal entry, it MUST be stored in the
+    DataStore.
     '''
     if os.environ.get('SUGAR_ACTIVITY_ROOT'):
         return os.environ['SUGAR_ACTIVITY_ROOT']
@@ -1498,7 +1565,7 @@ def get_activity_root():
         activity_root = env.get_profile_path(os.environ['SUGAR_BUNDLE_ID'])
         try:
             os.mkdir(activity_root)
-        except OSError, e:
+        except OSError as e:
             if e.errno != EEXIST:
                 raise e
         return activity_root
