@@ -19,6 +19,7 @@
 
 from gi.repository import Gdk
 from gi.repository import Gtk
+from gi.repository import GObject
 import gettext
 
 from sugar3.graphics.toolbutton import ToolButton
@@ -33,7 +34,8 @@ from sugar3.graphics.palettemenu import PaletteMenuBox
 from sugar3 import profile
 
 
-_ = lambda msg: gettext.dgettext('sugar-toolkit-gtk3', msg)
+def _(msg):
+    return gettext.dgettext('sugar-toolkit-gtk3', msg)
 
 
 def _create_activity_icon(metadata):
@@ -71,6 +73,7 @@ class ActivityToolbarButton(ToolbarButton):
 
     def __init__(self, activity, **kwargs):
         toolbar = ActivityToolbar(activity, orientation_left=True)
+        toolbar.connect('enter-key-press', lambda widget: self.emit('clicked'))
 
         ToolbarButton.__init__(self, page=toolbar, **kwargs)
 
@@ -167,6 +170,9 @@ class ShareButton(RadioMenuButton):
 
 
 class TitleEntry(Gtk.ToolItem):
+    __gsignals__ = {
+        'enter-key-press': (GObject.SignalFlags.RUN_FIRST, None, ([])),
+    }
 
     def __init__(self, activity, **kwargs):
         Gtk.ToolItem.__init__(self)
@@ -176,7 +182,8 @@ class TitleEntry(Gtk.ToolItem):
         self.entry.set_size_request(int(Gdk.Screen.width() / 3), -1)
         self.entry.set_text(activity.metadata['title'])
         self.entry.connect(
-            'focus-out-event', self.__title_changed_cb, activity)
+            'focus-out-event', self.__focus_out_event_cb, activity)
+        self.entry.connect('activate', self.__activate_cb, activity)
         self.entry.connect('button-press-event', self.__button_press_event_cb)
         self.entry.show()
         self.add(self.entry)
@@ -184,6 +191,14 @@ class TitleEntry(Gtk.ToolItem):
         activity.metadata.connect('updated', self.__jobject_updated_cb)
         activity.connect('_closing', self.__closing_cb)
 
+    def __activate_cb(self, entry, activity):
+        self.save_title(activity)
+        entry.select_region(0, 0)
+        entry.hide()
+        entry.show()
+        self.emit('enter-key-press')
+        return False
+
     def modify_bg(self, state, color):
         Gtk.ToolItem.modify_bg(self, state, color)
         self.entry.modify_bg(state, color)
@@ -199,7 +214,8 @@ class TitleEntry(Gtk.ToolItem):
         self.save_title(activity)
         return False
 
-    def __title_changed_cb(self, editable, event, activity):
+    def __focus_out_event_cb(self, widget, event, activity):
+        widget.select_region(0, 0)
         self.save_title(activity)
         return False
 
@@ -311,6 +327,9 @@ class DescriptionItem(ToolButton):
 
 class ActivityToolbar(Gtk.Toolbar):
     """The Activity toolbar with the Journal entry title and sharing button"""
+    __gsignals__ = {
+        'enter-key-press': (GObject.SignalFlags.RUN_FIRST, None, ([])),
+    }
 
     def __init__(self, activity, orientation_left=False):
         Gtk.Toolbar.__init__(self)
@@ -319,6 +338,8 @@ class ActivityToolbar(Gtk.Toolbar):
 
         if activity.metadata:
             title_button = TitleEntry(activity)
+            title_button.connect('enter-key-press',
+                                 lambda widget: self.emit('enter-key-press'))
             title_button.show()
             self.insert(title_button, -1)
             self.title = title_button.entry
