@@ -29,12 +29,18 @@ from sugar3.graphics.icon import Icon
 from sugar3.graphics.palette import Palette, ToolInvoker, WidgetInvoker
 
 
-_ = lambda msg: gettext.dgettext('sugar-toolkit-gtk3', msg)
+def _(msg):
+    return gettext.dgettext('sugar-toolkit-gtk3', msg)
+
+
+if not hasattr(GObject.ParamFlags, 'READWRITE'):
+    GObject.ParamFlags.READWRITE = GObject.ParamFlags.WRITABLE | \
+        GObject.ParamFlags.READABLE
 
 
 def get_svg_color_string(color):
-    return '#%.2X%.2X%.2X' % (color.red / 257, color.green / 257,
-                              color.blue / 257)
+    return '#%.2X%.2X%.2X' % (color.red // 257, color.green // 257,
+                              color.blue // 257)
 
 
 class _ColorButton(Gtk.Button):
@@ -77,8 +83,8 @@ class _ColorButton(Gtk.Button):
                              [Gtk.TargetEntry.new(
                                  'application/x-color', 0, 0)],
                              Gdk.DragAction.COPY)
-        self.connect('drag_data_received', self.__drag_data_received_cb)
-        self.connect('drag_data_get', self.__drag_data_get_cb)
+        self.connect('drag-data-received', self.__drag_data_received_cb)
+        self.connect('drag-data-get', self.__drag_data_get_cb)
 
         self._preview.fill_color = get_svg_color_string(self._color)
         self._preview.stroke_color = self._get_fg_style_color_str()
@@ -118,8 +124,9 @@ class _ColorButton(Gtk.Button):
         context = self.get_style_context()
         fg_color = context.get_color(Gtk.StateType.NORMAL)
         # the color components are stored as float values between 0.0 and 1.0
-        return '#%.2X%.2X%.2X' % (fg_color.red * 255, fg_color.green * 255,
-                                  fg_color.blue * 255)
+        return '#%.2X%.2X%.2X' % (int(fg_color.red * 255),
+                                  int(fg_color.green * 255),
+                                  int(fg_color.blue * 255))
 
     def set_color(self, color):
         assert isinstance(color, Gdk.Color)
@@ -138,17 +145,17 @@ class _ColorButton(Gtk.Button):
     def get_color(self):
         return self._color
 
-    color = GObject.property(type=object, getter=get_color, setter=set_color)
+    color = GObject.Property(type=object, getter=get_color, setter=set_color)
 
     def set_icon_name(self, icon_name):
         '''
         Sets the icon for the tool button from a named themed icon.
         If it is none then no icon will be shown.
-        
+
         Args:
             icon_name(string): The name for a themed icon.
             It can be set as 'None' too.
-        
+
         Example:
             set_icon_name('view-radial')
         '''
@@ -161,16 +168,16 @@ class _ColorButton(Gtk.Button):
         '''
         return self._preview.props.icon_name
 
-    icon_name = GObject.property(type=str,
+    icon_name = GObject.Property(type=str,
                                  getter=get_icon_name, setter=set_icon_name)
 
-    def set_icon_size(self, icon_size):
-        self._preview.props.icon_size = icon_size
+    def set_icon_size(self, pixel_size):
+        self._preview.props.pixel_size = pixel_size
 
     def get_icon_size(self):
-        return self._preview.props.icon_size
+        return self._preview.props.pixel_size
 
-    icon_size = GObject.property(type=int,
+    icon_size = GObject.Property(type=int,
                                  getter=get_icon_size, setter=set_icon_size)
 
     def set_title(self, title):
@@ -181,7 +188,7 @@ class _ColorButton(Gtk.Button):
     def get_title(self):
         return self._title
 
-    title = GObject.property(type=str, getter=get_title, setter=set_title)
+    title = GObject.Property(type=str, getter=get_title, setter=set_title)
 
     def _set_has_invoker(self, has_invoker):
         self._has_invoker = has_invoker
@@ -189,9 +196,9 @@ class _ColorButton(Gtk.Button):
     def _get_has_invoker(self):
         return self._has_invoker
 
-    has_invoker = GObject.property(type=bool, default=True,
-                                   flags=GObject.PARAM_READWRITE |
-                                   GObject.PARAM_CONSTRUCT_ONLY,
+    has_invoker = GObject.Property(type=bool, default=True,
+                                   flags=GObject.ParamFlags.READWRITE |
+                                   GObject.ParamFlags.CONSTRUCT_ONLY,
                                    getter=_get_has_invoker,
                                    setter=_set_has_invoker)
 
@@ -201,9 +208,9 @@ class _ColorButton(Gtk.Button):
     def _get_has_palette(self):
         return self._has_palette
 
-    has_palette = GObject.property(type=bool, default=True,
-                                   flags=GObject.PARAM_READWRITE |
-                                   GObject.PARAM_CONSTRUCT_ONLY,
+    has_palette = GObject.Property(type=bool, default=True,
+                                   flags=GObject.ParamFlags.READWRITE |
+                                   GObject.ParamFlags.CONSTRUCT_ONLY,
                                    getter=_get_has_palette,
                                    setter=_set_has_palette)
 
@@ -213,9 +220,9 @@ class _ColorButton(Gtk.Button):
     def _get_accept_drag(self):
         return self._accept_drag
 
-    accept_drag = GObject.property(type=bool, default=True,
-                                   flags=GObject.PARAM_READWRITE |
-                                   GObject.PARAM_CONSTRUCT_ONLY,
+    accept_drag = GObject.Property(type=bool, default=True,
+                                   flags=GObject.ParamFlags.READWRITE |
+                                   GObject.ParamFlags.CONSTRUCT_ONLY,
                                    getter=_get_accept_drag,
                                    setter=_set_accept_drag)
 
@@ -412,7 +419,7 @@ class _ColorPalette(Palette):
     def get_color(self):
         return self._color
 
-    color = GObject.property(type=object, getter=get_color, setter=set_color)
+    color = GObject.Property(type=object, getter=get_color, setter=set_color)
 
 
 def _add_accelerator(tool_button):
@@ -491,13 +498,13 @@ class ColorToolButton(Gtk.ToolItem):
     def set_accelerator(self, accelerator):
         '''
         Sets keyboard shortcut that activates this button.
-        
+
         Args:
             accelerator(string): accelerator to be set. Should be in
             form <modifier>Letter
             Find about format here :
             https://developer.gnome.org/gtk3/stable/gtk3-Keyboard-Accelerators.html#gtk-accelerator-parse
-        
+
         Example:
             set_accelerator(self, 'accel')
         '''
@@ -510,7 +517,7 @@ class ColorToolButton(Gtk.ToolItem):
         '''
         return self._accelerator
 
-    accelerator = GObject.property(type=str, setter=set_accelerator,
+    accelerator = GObject.Property(type=str, setter=set_accelerator,
                                    getter=get_accelerator)
 
     def create_palette(self):
@@ -518,21 +525,21 @@ class ColorToolButton(Gtk.ToolItem):
         The create_palette function is called when the palette needs to be
         invoked.  For example, when the user has right clicked the icon or
         the user has hovered over the icon for a long time.
-        
+
         The create_palette will only be called once or zero times.  The palette
         returned will be stored and re-used if the user invokes the palette
         multiple times.
-        
+
         Your create_palette implementation does not need to
         :any:`Gtk.Widget.show` the palette, as this will be done by the
         invoker.  However, you still need to show
         the menu items, etc that you place in the palette.
-        
+
         Returns:
-            
+
             sugar3.graphics.palette.Palette, or None to indicate that you
             do not want a palette shown
-        
+
         The default implementation returns None, to indicate no palette should
         be shown.
         '''
@@ -546,7 +553,7 @@ class ColorToolButton(Gtk.ToolItem):
         self._palette_invoker.detach()
         self._palette_invoker = palette_invoker
 
-    palette_invoker = GObject.property(
+    palette_invoker = GObject.Property(
         type=object, setter=set_palette_invoker, getter=get_palette_invoker)
 
     def set_expanded(self, expanded):
@@ -584,17 +591,17 @@ class ColorToolButton(Gtk.ToolItem):
         '''
         return self.get_child().props.color
 
-    color = GObject.property(type=object, getter=get_color, setter=set_color)
+    color = GObject.Property(type=object, getter=get_color, setter=set_color)
 
     def set_icon_name(self, icon_name):
         '''
         Sets the icon for the tool button from a named themed icon.
         If it is none then no icon will be shown.
-        
+
         Args:
             icon_name(string): The name for a themed icon.
             It can be set as 'None' too.
-        
+
         Example:
             set_icon_name('view-radial')
         '''
@@ -607,7 +614,7 @@ class ColorToolButton(Gtk.ToolItem):
         '''
         return self.get_child().props.icon_name
 
-    icon_name = GObject.property(type=str,
+    icon_name = GObject.Property(type=str,
                                  getter=get_icon_name, setter=set_icon_name)
 
     def set_icon_size(self, icon_size):
@@ -622,13 +629,13 @@ class ColorToolButton(Gtk.ToolItem):
         '''
         return self.get_child().props.icon_size
 
-    icon_size = GObject.property(type=int,
+    icon_size = GObject.Property(type=int,
                                  getter=get_icon_size, setter=set_icon_size)
 
     def set_title(self, title):
         '''
-        The set_title() method sets the "title" property to the value of 
-        title. The "title" property contains the string that is used to 
+        The set_title() method sets the "title" property to the value of
+        title. The "title" property contains the string that is used to
         set the colorbutton title.
         '''
         self.get_child().props.title = title
@@ -639,7 +646,7 @@ class ColorToolButton(Gtk.ToolItem):
         '''
         return self.get_child().props.title
 
-    title = GObject.property(type=str, getter=get_title, setter=set_title)
+    title = GObject.Property(type=str, getter=get_title, setter=set_title)
 
     def do_draw(self, cr):
         if self._palette and self._palette.is_up():
