@@ -24,6 +24,7 @@ gi.require_version('Gtk', '3.0')
 from gi.repository import Gio
 from gi.repository import Gtk
 from gi.repository import Gdk
+from gi.repository import GLib
 from gi.repository import GObject
 
 _HAS_GST = True
@@ -32,7 +33,7 @@ try:
     from gi.repository import Gst
     Gst.init(None)
     Gst.parse_launch('espeak')
-except:
+except BaseException:
     logging.error('Gst or the espeak plugin is not installed in the system.')
     _HAS_GST = False
 
@@ -71,7 +72,7 @@ translated_espeak_voices = {
     'de': _('German'),
     # Translators:  http://en.wikipedia.org/wiki/Modern_Greek
     'el': _('Greek'),
-    'en': _('Default'),
+    'en': _('English'),
     # Translators:  http://en.wikipedia.org/wiki/British_English
     'en_gb': _('English Britain'),
     # Translators:  http://en.wikipedia.org/wiki/Scottish_English
@@ -192,7 +193,8 @@ class SpeechManager(GObject.GObject):
     __gsignals__ = {
         'play': (GObject.SignalFlags.RUN_FIRST, None, []),
         'pause': (GObject.SignalFlags.RUN_FIRST, None, []),
-        'stop': (GObject.SignalFlags.RUN_FIRST, None, [])
+        'stop': (GObject.SignalFlags.RUN_FIRST, None, []),
+        'mark': (GObject.SignalFlags.RUN_FIRST, None, [str])
     }
 
     MIN_PITCH = -100
@@ -203,15 +205,16 @@ class SpeechManager(GObject.GObject):
 
     def __init__(self, **kwargs):
         GObject.GObject.__init__(self, **kwargs)
-        self._player = None
+        self.player = None
         if not self.enabled():
             return
 
-        self._player = _GstSpeechPlayer()
-        self._player.connect('play', self._update_state, 'play')
-        self._player.connect('stop', self._update_state, 'stop')
-        self._player.connect('pause', self._update_state, 'pause')
-        self._default_voice_name = self._player.get_default_voice()
+        self.player = GstSpeechPlayer()
+        self.player.connect('play', self._update_state, 'play')
+        self.player.connect('stop', self._update_state, 'stop')
+        self.player.connect('pause', self._update_state, 'pause')
+        self.player.connect('mark', self._mark_cb)
+        self._default_voice_name = self.player.get_default_voice()
         self._pitch = DEFAULT_PITCH
         self._rate = DEFAULT_RATE
         self._is_playing = False
@@ -227,16 +230,19 @@ class SpeechManager(GObject.GObject):
         self._is_paused = (signal == 'pause')
         self.emit(signal)
 
+    def _mark_cb(self, player, value):
+        self.emit('mark', value)
+
     def get_is_playing(self):
         return self._is_playing
 
-    is_playing = GObject.property(type=bool, getter=get_is_playing,
+    is_playing = GObject.Property(type=bool, getter=get_is_playing,
                                   setter=None, default=False)
 
     def get_is_paused(self):
         return self._is_paused
 
-    is_paused = GObject.property(type=bool, getter=get_is_paused,
+    is_paused = GObject.Property(type=bool, getter=get_is_paused,
                                  setter=None, default=False)
 
     def get_pitch(self):
@@ -248,14 +254,14 @@ class SpeechManager(GObject.GObject):
     def set_pitch(self, pitch):
         self._pitch = pitch
         if self._save_timeout_id != -1:
-            GObject.source_remove(self._save_timeout_id)
-        self._save_timeout_id = GObject.timeout_add(_SAVE_TIMEOUT, self.save)
+            GLib.source_remove(self._save_timeout_id)
+        self._save_timeout_id = GLib.timeout_add(_SAVE_TIMEOUT, self.save)
 
     def set_rate(self, rate):
         self._rate = rate
         if self._save_timeout_id != -1:
-            GObject.source_remove(self._save_timeout_id)
-        self._save_timeout_id = GObject.timeout_add(_SAVE_TIMEOUT, self.save)
+            GLib.source_remove(self._save_timeout_id)
+        self._save_timeout_id = GLib.timeout_add(_SAVE_TIMEOUT, self.save)
 
     def say_text(self, text, pitch=None, rate=None, lang_code=None):
         if pitch is None:
@@ -265,34 +271,34 @@ class SpeechManager(GObject.GObject):
         if lang_code is None:
             voice_name = self._default_voice_name
         else:
-            voice_name = self._player.get_all_voices()[lang_code]
+            voice_name = self.player.get_all_voices()[lang_code]
         if text:
-            logging.debug('PLAYING "%s" lang %s', text, voice_name)
-            self._player.speak(pitch, rate, voice_name, text)
+            logging.error(
+                'PLAYING %r lang %r pitch %r rate %r',
+                text,
+                voice_name,
+                pitch,
+                rate)
+            self.player.speak(pitch, rate, voice_name, text)
 
     def say_selected_text(self):
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
         clipboard.request_text(self.__primary_selection_cb, None)
 
     def pause(self):
-        self._player.pause_sound_device()
+        self.player.pause_sound_device()
 
     def restart(self):
-        self._player.restart_sound_device()
+        self.player.restart_sound_device()
 
     def stop(self):
-        self._player.stop_sound_device()
+        self.player.stop_sound_device()
 
     def __primary_selection_cb(self, clipboard, text, user_data):
         self.say_text(text)
 
     def save(self):
         self._save_timeout_id = -1
-        # DEPRECATED
-        from gi.repository import GConf
-        client = GConf.Client.get_default()
-        client.set_int('/desktop/sugar/speech/pitch', self._pitch)
-        client.set_int('/desktop/sugar/speech/rate', self._rate)
 
         settings = Gio.Settings('org.sugarlabs.speech')
         settings.set_int('pitch', self._pitch)
@@ -309,78 +315,83 @@ class SpeechManager(GObject.GObject):
                       self._pitch, self._rate)
 
     def get_all_voices(self):
-        if self._player:
-            return self._player.get_all_voices()
+        if self.player:
+            return self.player.get_all_voices()
         return None
 
     def get_all_traslated_voices(self):
         """ deprecated after 0.112, due to method name spelling error """
-        if self._player:
-            return self._player.get_all_translated_voices()
+        if self.player:
+            return self.player.get_all_translated_voices()
         return None
 
     def get_all_translated_voices(self):
-        if self._player:
-            return self._player.get_all_translated_voices()
+        if self.player:
+            return self.player.get_all_translated_voices()
         return None
 
 
-class _GstSpeechPlayer(GObject.GObject):
+class GstSpeechPlayer(GObject.GObject):
 
     __gsignals__ = {
         'play': (GObject.SignalFlags.RUN_FIRST, None, []),
         'pause': (GObject.SignalFlags.RUN_FIRST, None, []),
-        'stop': (GObject.SignalFlags.RUN_FIRST, None, [])
+        'stop': (GObject.SignalFlags.RUN_FIRST, None, []),
+        'mark': (GObject.SignalFlags.RUN_FIRST, None, [str])
     }
 
     def __init__(self):
         GObject.GObject.__init__(self)
-        self._pipeline = None
+        self.pipeline = None
         self._all_voices = None
         self._all_translated_voices = None
 
     def restart_sound_device(self):
-        if self._pipeline is None:
+        if self.pipeline is None:
             logging.debug('Trying to restart not initialized sound device')
             return
 
         power.get_power_manager().inhibit_suspend()
-        self._pipeline.set_state(Gst.State.PLAYING)
+        self.pipeline.set_state(Gst.State.PLAYING)
         self.emit('play')
 
     def pause_sound_device(self):
-        if self._pipeline is None:
+        if self.pipeline is None:
             return
 
-        self._pipeline.set_state(Gst.State.PAUSED)
+        self.pipeline.set_state(Gst.State.PAUSED)
         power.get_power_manager().restore_suspend()
         self.emit('pause')
 
     def stop_sound_device(self):
-        if self._pipeline is None:
+        if self.pipeline is None:
             return
 
-        self._pipeline.set_state(Gst.State.NULL)
+        self.pipeline.set_state(Gst.State.NULL)
         power.get_power_manager().restore_suspend()
         self.emit('stop')
 
     def make_pipeline(self, command):
-        if self._pipeline is not None:
+        if self.pipeline is not None:
             self.stop_sound_device()
-            del self._pipeline
+            del self.pipeline
 
-        self._pipeline = Gst.parse_launch(command)
+        self.pipeline = Gst.parse_launch(command)
 
-        bus = self._pipeline.get_bus()
+        bus = self.pipeline.get_bus()
         bus.add_signal_watch()
         bus.connect('message', self.__pipe_message_cb)
 
     def __pipe_message_cb(self, bus, message):
         if message.type in (Gst.MessageType.EOS, Gst.MessageType.ERROR):
-            self._pipeline.set_state(Gst.State.NULL)
-            self._pipeline = None
+            self.pipeline.set_state(Gst.State.NULL)
+            self.pipeline = None
             power.get_power_manager().restore_suspend()
             self.emit('stop')
+        elif message.type is Gst.MessageType.ELEMENT and \
+                message.get_structure().get_name() == 'espeak-mark':
+            mark_value = message.get_structure().get_value('mark')
+            self.emit('mark', mark_value)
 
     def speak(self, pitch, rate, voice_name, text):
         # TODO workaround for http://bugs.sugarlabs.org/ticket/1801
@@ -388,7 +399,7 @@ class _GstSpeechPlayer(GObject.GObject):
             return
 
         self.make_pipeline('espeak name=espeak ! autoaudiosink')
-        src = self._pipeline.get_by_name('espeak')
+        src = self.pipeline.get_by_name('espeak')
 
         src.props.text = text
         src.props.pitch = pitch
@@ -441,5 +452,5 @@ class _GstSpeechPlayer(GObject.GObject):
             language_location = 'es_la'
 
         best = voices.get(language_location) or voices.get(language) \
-            or 'default'
+            or 'english'
         return best
